@@ -1,10 +1,9 @@
 # command line interface
 
 import asyncclick as click
-import yaml
 import sys
 
-from distkv.util import yprint, NotGiven
+from distkv.util import yprint, NotGiven, yload, P
 
 import logging
 
@@ -18,20 +17,16 @@ async def cli():
 
 
 @cli.command()
-@click.option(
-    "-s", "--script", type=click.File(mode="w", lazy=True), help="Save the code here"
-)
-@click.argument("path", nargs=-1)
+@click.option("-s", "--script", type=click.File(mode="w", lazy=True), help="Save the code here")
+@click.argument("path", nargs=1)
 @click.pass_obj
 async def get(obj, path, script):
     """Read a code entry"""
-    if not path:
+    path = P(path)
+    if not len(path):
         raise click.UsageError("You need a non-empty path.")
     res = await obj.client._request(
-        action="get_value",
-        path=obj.cfg["codes"]["prefix"] + path,
-        iter=False,
-        nchain=obj.meta,
+        action="get_value", path=obj.cfg["codes"]["prefix"] + path, iter=False, nchain=obj.meta
     )
     if "value" not in res:
         if obj.debug:
@@ -48,18 +43,12 @@ async def get(obj, path, script):
 
 @cli.command("set")
 @click.option("-a", "--async", "async_", is_flag=True, help="The code is async")
-@click.option(
-    "-t", "--thread", is_flag=True, help="The code should run in a worker thread"
-)
+@click.option("-t", "--thread", is_flag=True, help="The code should run in a worker thread")
 @click.option("-s", "--script", type=click.File(mode="r"), help="File with the code")
-@click.option(
-    "-v", "--vars", "vars_", multiple=True, type=str, help="Required variables"
-)
+@click.option("-v", "--vars", "vars_", multiple=True, type=str, help="Required variables")
 @click.option("-i", "--info", type=str, help="one-liner info about the code")
-@click.option(
-    "-d", "--data", type=click.File(mode="r"), help="load the metadata (YAML)"
-)
-@click.argument("path", nargs=-1)
+@click.option("-d", "--data", type=click.File(mode="r"), help="load the metadata (YAML)")
+@click.argument("path", nargs=1)
 @click.pass_obj
 async def set_(obj, path, thread, script, data, vars_, async_, info):
     """Save Python code."""
@@ -72,11 +61,12 @@ async def set_(obj, path, thread, script, data, vars_, async_, info):
         else:
             async_ = None
 
-    if not path:
+    path = P(path)
+    if not len(path):
         raise click.UsageError("You need a non-empty path.")
 
     if data:
-        msg = yaml.safe_load(data)
+        msg = yload(data)
     else:
         msg = {}
     chain = NotGiven
@@ -124,20 +114,16 @@ async def mod():
 
 
 @mod.command("get")
-@click.option(
-    "-s", "--script", type=click.File(mode="w", lazy=True), help="Save the code here"
-)
-@click.argument("path", nargs=-1)
+@click.option("-s", "--script", type=click.File(mode="w", lazy=True), help="Save the code here")
+@click.argument("path", nargs=1)
 @click.pass_obj  # pylint: disable=function-redefined
 async def get_mod(obj, path, script):
     """Read a module entry"""
-    if not path:
+    path = P(path)
+    if not len(path):
         raise click.UsageError("You need a non-empty path.")
     res = await obj.client._request(
-        action="get_value",
-        path=obj.cfg["modules"]["prefix"] + path,
-        iter=False,
-        nchain=obj.meta,
+        action="get_value", path=obj.cfg["modules"]["prefix"] + path, iter=False, nchain=obj.meta
     )
     if not obj.meta:
         res = res.value
@@ -154,21 +140,18 @@ async def get_mod(obj, path, script):
 
 
 @mod.command("set")
-@click.option(
-    "-s", "--script", type=click.File(mode="r"), help="File with the module's code"
-)
-@click.option(
-    "-d", "--data", type=click.File(mode="r"), help="load the metadata (YAML)"
-)
-@click.argument("path", nargs=-1)  # pylint: disable=function-redefined
+@click.option("-s", "--script", type=click.File(mode="r"), help="File with the module's code")
+@click.option("-d", "--data", type=click.File(mode="r"), help="load the metadata (YAML)")
+@click.argument("path", nargs=1)  # pylint: disable=function-redefined
 @click.pass_obj
 async def set_mod(obj, path, script, data):
     """Save a Python module to DistKV."""
-    if not path:
+    path = P(path)
+    if not len(path):
         raise click.UsageError("You need a non-empty path.")
 
     if data:
-        msg = yaml.safe_load(data)
+        msg = yload(data)
     else:
         msg = {}
     chain = None
@@ -185,12 +168,7 @@ async def set_mod(obj, path, script, data):
         msg["code"] = script.read()
 
     res = await obj.client.set(
-        *obj.cfg["modules"]["prefix"],
-        *path,
-        value=msg,
-        iter=False,
-        nchain=obj.meta,
-        chain=chain,
+        *obj.cfg["modules"]["prefix"], *path, value=msg, iter=False, nchain=obj.meta, chain=chain
     )
     if obj.meta:
         yprint(res, stream=obj.stdout)
@@ -205,22 +183,14 @@ async def set_mod(obj, path, script, data):
     "for values. Default: return as list",
 )
 @click.option(
-    "-m",
-    "--maxdepth",
-    type=int,
-    default=None,
-    help="Limit recursion depth. Default: whole tree",
+    "-m", "--maxdepth", type=int, default=None, help="Limit recursion depth. Default: whole tree"
 )
 @click.option(
-    "-M",
-    "--mindepth",
-    type=int,
-    default=None,
-    help="Starting depth. Default: whole tree",
+    "-M", "--mindepth", type=int, default=None, help="Starting depth. Default: whole tree"
 )
 @click.option("-f", "--full", is_flag=True, help="print complete entries.")
 @click.option("-s", "--short", is_flag=True, help="print shortened entries.")
-@click.argument("path", nargs=-1)
+@click.argument("path", nargs=1)
 @click.pass_obj
 async def list_(obj, path, as_dict, maxdepth, mindepth, full, short):
     """
@@ -231,6 +201,7 @@ async def list_(obj, path, as_dict, maxdepth, mindepth, full, short):
     for incremental output.
     """
 
+    path = P(path)
     if (full or as_dict) and short:
         raise click.UsageError("'-f'/'-d' and '-s' are incompatible.")
     kw = {}
@@ -239,14 +210,12 @@ async def list_(obj, path, as_dict, maxdepth, mindepth, full, short):
     if mindepth is not None:
         kw["min_depth"] = mindepth
     y = {}
-    async for r in obj.client.get_tree(
-        *obj.cfg["codes"].prefix, *path, nchain=obj.meta, **kw
-    ):
+    async for r in obj.client.get_tree(*obj.cfg["codes"].prefix, *path, nchain=obj.meta, **kw):
         r.pop("seq", None)
         path = r.pop("path")
         if not full:
             if "info" not in r.value:
-                r.value.info = "<%d lines>" % (len(r.value.code.splitlines()),)
+                r.value.info = f"<{len(r.value.code.splitlines())} lines>"
             del r.value["code"]
 
         if short:
