@@ -1,9 +1,10 @@
 # command line interface
 
 import asyncclick as click
+import datetime
 
 from distkv.errors import ErrorRoot
-from distkv.util import yprint
+from distkv.util import yprint, P
 
 import logging
 
@@ -19,20 +20,17 @@ async def cli(obj):
 
 @cli.command()
 @click.option("-n", "--node", help="add details from this node")
-@click.option(
-    "-s", "--subsystem", "subsys", help="only show errors from this subsystem"
-)
+@click.option("-s", "--subsystem", "subsys", help="only show errors from this subsystem")
 @click.option("-r", "--resolved", is_flag=True, help="only resolved errors")
 @click.option("-t", "--trace", is_flag=True, help="add traces, if present")
 @click.option("-a", "--all-nodes", is_flag=True, help="add details from all nodes")
-@click.option(
-    "-d", "--as-dict", default=None, help="Dump a list of all open (or resolved) error."
-)
-@click.argument("path", nargs=-1)
+@click.option("-d", "--as-dict", default=None, help="Dump a list of all open (or resolved) error.")
+@click.argument("path", nargs=1)
 @click.pass_obj
 async def dump(obj, as_dict, path, node, all_nodes, trace, resolved, subsys):
     """Dump error entries.
     """
+    path = P(path)
     path_ = obj.cfg["errors"].prefix
     d = 1
     if node is not None:
@@ -47,6 +45,15 @@ async def dump(obj, as_dict, path, node, all_nodes, trace, resolved, subsys):
             return
         if resolved == (not val.get("resolved", False)):
             return
+        fs = val.get("first_seen")
+        if fs:
+            val.first_seen_date = datetime.datetime.fromtimestamp(fs).strftime("%Y-%m-%d %H:%M:%S")
+        ls = val.get("last_seen")
+        if ls:
+            val.last_seen_date = datetime.datetime.fromtimestamp(ls).strftime("%Y-%m-%d %H:%M:%S")
+        fs = val.get("seen")
+        if fs:
+            val["seen_date"] = datetime.datetime.fromtimestamp(fs).strftime("%Y-%m-%d %H:%M:%S")
         try:
             rp = val.path
             if as_dict:
@@ -74,6 +81,12 @@ async def dump(obj, as_dict, path, node, all_nodes, trace, resolved, subsys):
                 nchain=3 if obj.meta else 0,
             )
             async for rr in rs:
+                val = rr.value
+                fs = val.get("seen")
+                if fs:
+                    val["seen_date"] = datetime.datetime.fromtimestamp(fs).strftime(
+                        "%Y-%m-%d %H:%M:%S"
+                    )
                 if not all_nodes:
                     try:
                         rn[rr.path[-1]] = rr.value.trace
@@ -105,11 +118,13 @@ async def dump(obj, as_dict, path, node, all_nodes, trace, resolved, subsys):
 
     if node is not None and len(path) == 1:  # single error?
         try:
-            tock = int(path[0])
+            if len(path) != 2:
+                raise ValueError
+            tock = int(path[1])
         except ValueError:
             pass
         else:
-            r = await obj.client.get(*path_, tock, nchain=3 if obj.meta else 0)
+            r = await obj.client.get(path_, tock, nchain=3 if obj.meta else 0)
             # Mangle a few variables so that the output is still OK
             path = ()
             path_ = path_[:-1]
@@ -122,9 +137,7 @@ async def dump(obj, as_dict, path, node, all_nodes, trace, resolved, subsys):
             res = ait(r)
 
     if res is None:
-        res = obj.client.get_tree(
-            *path_, min_depth=d, max_depth=d, nchain=3 if obj.meta else 0
-        )
+        res = obj.client.get_tree(path_, min_depth=d, max_depth=d, nchain=3 if obj.meta else 0)
     async for r in res:
         await one(r)
 
