@@ -86,7 +86,7 @@ from weakref import WeakValueDictionary
 from time import time  # wall clock, intentionally
 
 from .obj import AttrClientEntry, ClientEntry, ClientRoot
-from .util import Cache, NotGiven
+from .util import Cache, NotGiven, Path
 from .codec import packer
 from .exceptions import ServerError
 
@@ -107,11 +107,7 @@ class ErrorSubEntry(AttrClientEntry):
         return ClientEntry
 
     def __repr__(self):
-        return "‹%s %s %d›" % (
-            self.__class__.__name__,
-            self._path[-3:],
-            getattr(self, "tock", -1),
-        )
+        return "‹%s %s %d›" % (self.__class__.__name__, self._path[-3:], getattr(self, "tock", -1))
 
     async def set_value(self, value):
         await super().set_value(value)
@@ -129,6 +125,7 @@ class ErrorEntry(AttrClientEntry):
 
     ATTRS = "path subsystem severity resolved created count first_seen last_seen message".split()
     resolved = None  # bool; if None, no details yet
+    created = None
     count = 0
     subsystem = None
     path = ()
@@ -212,7 +209,7 @@ class ErrorEntry(AttrClientEntry):
             logger.warning("Error %r %s: %s", self._path, node, m)
 
         try:
-            await res.save()
+            r = await res.save()
         except TypeError:
             for k in res.ATTRS:
                 v = getattr(res, k, None)
@@ -220,21 +217,17 @@ class ErrorEntry(AttrClientEntry):
                     packer(v)
                 except TypeError:
                     setattr(res, k, repr(v))
-            await res.save()
+            r = await res.save()
+        return r
 
     async def add_comment(self, node, comment, data):
         """
         Store this comment, typically used when something resumes working.
         One per node, so we don't need to avoid collisions.
         """
-        res = dict(
-            seen=time(),
-            tock=await self.root.client.get_tock(),
-            comment=comment,
-            data=data,
-        )
+        res = dict(seen=time(), tock=await self.root.client.get_tock(), comment=comment, data=data)
         logger.info("Comment %s: %s", node, comment)
-        await self.root.client.set(*self._path, chain=self.chain, value=res)
+        await self.root.client.set(self._path, chain=self.chain, value=res)
 
     async def delete(self):  # pylint: disable=signature-differs,arguments-differ
         """
@@ -288,9 +281,7 @@ class ErrorEntry(AttrClientEntry):
         if value is NotGiven:
             if self.value is NotGiven:
                 return
-            keep = await self.root.get_error_record(
-                self.subsystem, *self.path, create=False
-            )
+            keep = await self.root.get_error_record(self.subsystem, self.path, create=False)
             if keep is not None:
                 self._real_entry = keep.real_entry
                 await self.move_to_real()
@@ -380,7 +371,7 @@ class ErrorRoot(ClientRoot):
         else:
             yield from iter(self._active[subsystem].values())
 
-    async def get_error_record(self, subsystem, *path, create=True):
+    async def get_error_record(self, subsystem, path, *, create=True):
         """Retrieve or generate an error record for a particular subsystem
         and path.
 
@@ -397,7 +388,7 @@ class ErrorRoot(ClientRoot):
         if not create:
             return None
         tock = await self.client.get_tock()
-        return self.follow(self.name, tock, create=True)
+        return self.follow(Path(self.name, tock), create=True)
 
     async def _unique(self, entry):
         """
@@ -413,7 +404,7 @@ class ErrorRoot(ClientRoot):
         not depend on which node it is running on or which entry arrives
         first.
         """
-        other = await self.get_error_record(entry.subsystem, *entry.path, create=False)
+        other = await self.get_error_record(entry.subsystem, entry.path, create=False)
         if other is None or other is entry:
             return None, None
 
@@ -427,10 +418,10 @@ class ErrorRoot(ClientRoot):
         elif other.node < entry.node:
             return entry, other
 
-        raise RuntimeError("This cannot happen: %s %s" % (entry.node, entry.tock))
+        raise RuntimeError(f"This cannot happen: {entry.node} {entry.tock}")
 
     async def record_working(  # pylint: disable=dangerous-default-value
-        self, subsystem, *path, comment=None, data={}, force=False
+        self, subsystem, path, *, comment=None, data={}, force=False
     ):
         """This exception has been fixed.
 
@@ -441,7 +432,7 @@ class ErrorRoot(ClientRoot):
           data (dict): any relevant data
           force (bool): create an entry even if no error is open.
         """
-        rec = await self.get_error_record(subsystem, *path, create=force)
+        rec = await self.get_error_record(subsystem, path, create=force)
         if rec is None:
             return
         if not rec.resolved:
@@ -454,13 +445,14 @@ class ErrorRoot(ClientRoot):
     async def record_error(  # pylint: disable=dangerous-default-value
         self,
         subsystem,
-        *path,
+        path,
+        *,
         exc=None,
         data={},
         severity=0,
         message=None,
         force: bool = False,
-        comment: str = None
+        comment: str = None,
     ):
         """An exception has occurred for this subtype and path.
 
@@ -475,44 +467,39 @@ class ErrorRoot(ClientRoot):
           message (str): some text to add to the error. It is formatted
             with the data when printed.
         """
-        rec = await self.get_error_record(subsystem, *path)
-        try:
-            if not force and rec.severity < severity:
-                return
-        except AttributeError:
-            pass
+        rec = await self.get_error_record(subsystem, path)
+        if not force and hasattr(rec, "severity") and rec.severity < severity:
+            return
 
         rec.severity = severity
         rec.subsystem = subsystem
         rec.path = path
         rec.resolved = False
         rec.count += 1
-        if not hasattr(rec, "first_seen"):
-            rec.first_seen = time()
         rec.last_seen = time()
+        if not hasattr(rec, "first_seen"):
+            rec.first_seen = rec.last_seen
 
         try:
             await rec.save()
         except anyio.exceptions.ClosedResourceError:
             return  # owch, but can't be helped
 
-        await rec.real_entry.add_exc(
+        r = await rec.real_entry.add_exc(
             self.name, exc=exc, data=data, comment=comment, message=message
         )
-        return rec
+        return r
 
     async def _pop(self, entry):
         """Override to deal with entry changes"""
         if entry.subsystem is None or entry.path is None:
             return
-        rec = await self.get_error_record(entry.subsystem, *entry.path, create=False)
+        rec = await self.get_error_record(entry.subsystem, entry.path, create=False)
         if rec is not entry:
             return
 
         try:
-            del (self._done if entry.resolved else self._active)[entry.subsystem][
-                entry.path
-            ]
+            del (self._done if entry.resolved else self._active)[entry.subsystem][entry.path]
         except KeyError:
             pass
 
