@@ -10,11 +10,15 @@ import anyio
 
 try:
     from trio import BrokenResourceError as trioBrokenResourceError
+    from trio import Cancelled as trioCancelled
 except ImportError:
 
     class trioBrokenResourceError(Exception):
         pass
 
+    class trioCancelled(BaseException):
+        pass
+
 
 try:
     from contextlib import asynccontextmanager
@@ -49,9 +53,12 @@ from .util import (
     byte2num,
     NotGiven,
     ValueEvent,
+    Path,
+    P,
 )
 from .exceptions import (
     ClientError,
+    ClientChainError,
     NoAuthError,
     CancelledError,
     ACLError,
@@ -185,12 +192,10 @@ class StreamCommand:
                     await self.send(**res)
             except Exception as exc:
                 if not isinstance(exc, CancelledError):
-                    self.client.logger.exception(
-                        "ERS%d %r", self.client._client_nr, self.msg
-                    )
+                    self.client.logger.exception("ERS%d %r", self.client._client_nr, self.msg)
                 await self.send(error=repr(exc))
             finally:
-                async with anyio.fail_after(2, shield=True):
+                async with anyio.move_on_after(2, shield=True):
                     await self.send(state="end")
 
         else:
@@ -249,14 +254,14 @@ class SCmd_auth(StreamCommand):
             await client._user.auth_sub(msg)
             return
 
-        root = msg.get("root", ())
-        auth = client.root.follow(*root, None, "auth", nulls_ok=2, create=False)
+        root = msg.get("root", Path())
+        auth = client.root.follow(root + (None, "auth"), nulls_ok=2, create=False)
         if client.user is None:
             a = auth.data["current"]
             if msg.typ != a and client.user is None:
                 raise RuntimeError("Wrong auth type", a)
 
-        data = auth.follow(msg.typ, "user", msg.ident, create=False)
+        data = auth.follow(Path(msg.typ, "user", msg.ident), create=False)
 
         cls = loader(msg.typ, "user", server=True)
         user = cls.load(data)
@@ -313,13 +318,13 @@ class SCmd_auth_list(StreamCommand):
             raise RuntimeError("Cannot read tenant users")
         kind = msg.get("kind", "user")
 
-        auth = client.root.follow(*root, None, "auth", nulls_ok=2, create=False)
+        auth = client.root.follow(root + (None, "auth"), nulls_ok=2, create=False)
         if "ident" in msg:
-            data = auth.follow(msg.typ, kind, msg.ident, create=False)
+            data = auth.follow(Path(msg.typ, kind, msg.ident), create=False)
             await self.send_one(data, nchain=nchain)
 
         else:
-            d = auth.follow(msg.typ, kind, create=False)
+            d = auth.follow(Path(msg.typ, kind), create=False)
             for data in d.values():
                 await self.send_one(data, nchain=nchain)
 
@@ -358,8 +363,8 @@ class SCmd_auth_get(StreamCommand):
             raise RuntimeError("Cannot read tenant users")
         kind = msg.get("kind", "user")
 
-        auth = client.root.follow(*root, None, "auth", nulls_ok=2, create=False)
-        data = auth.follow(msg.typ, kind, msg.ident, create=False)
+        auth = client.root.follow(root + (None, "auth"), nulls_ok=2, create=False)
+        data = auth.follow(Path(msg.typ, kind, msg.ident), create=False)
         cls = loader(msg.typ, kind, server=True, make=False)
         user = cls.load(data)
 
@@ -399,9 +404,9 @@ class SCmd_auth_set(StreamCommand):
         kind = msg.get("kind", "user")
 
         cls = loader(msg.typ, kind, server=True, make=True)
-        auth = client.root.follow(*root, None, "auth", nulls_ok=2, create=True)
+        auth = client.root.follow(root + (None, "auth"), nulls_ok=2, create=True)
 
-        data = auth.follow(msg.typ, kind, msg.ident, create=True)
+        data = auth.follow(Path(msg.typ, kind, msg.ident), create=True)
         user = cls.load(data)
         val = user.save()
         val = drop_dict(val, msg.pop("drop", ()))
@@ -434,16 +439,10 @@ class SCmd_get_tree(StreamCommand):
         if root is None:
             root = client.root
             entry, acl = root.follow_acl(
-                *msg.path,
-                create=False,
-                nulls_ok=client.nulls_ok,
-                acl=client.acl,
-                acl_key="e",
+                msg.path, create=False, nulls_ok=client.nulls_ok, acl=client.acl, acl_key="e"
             )
         else:
-            entry, _ = root.follow_acl(
-                *msg.path, create=False, nulls_ok=client.nulls_ok
-            )
+            entry, _ = root.follow_acl(msg.path, create=False, nulls_ok=client.nulls_ok)
             acl = NullACL
 
         kw = {}
@@ -504,11 +503,7 @@ class SCmd_watch(StreamCommand):
         client = self.client
         conv = client.conv
         entry, acl = client.root.follow_acl(
-            *msg.path,
-            acl=client.acl,
-            acl_key="x",
-            create=True,
-            nulls_ok=client.nulls_ok,
+            msg.path, acl=client.acl, acl_key="x", create=True, nulls_ok=client.nulls_ok
         )
         nchain = msg.get("nchain", 0)
         max_depth = msg.get("max_depth", -1)
@@ -529,9 +524,7 @@ class SCmd_watch(StreamCommand):
                                 return
                             if entry.tock < tock:
                                 res = entry.serialize(
-                                    chop_path=client._chop_path,
-                                    nchain=nchain,
-                                    conv=conv,
+                                    chop_path=client._chop_path, nchain=nchain, conv=conv
                                 )
                                 shorter(res)
                                 if not acl.allows("r"):
@@ -624,7 +617,7 @@ class ServerClient:
     def __init__(self, server: "Server", stream: anyio.abc.Stream):
         self.server = server
         self.root = server.root
-        self.metaroot = self.root.follow(None, create=True, nulls_ok=True)
+        self.metaroot = self.root.follow(Path(None), create=True, nulls_ok=True)
         self.stream = stream
         self.seq = 0
         self.tasks = {}
@@ -701,7 +694,7 @@ class ServerClient:
     def _chroot(self, root):
         if not root:
             return
-        entry, _acl = self.root.follow_acl(*root, acl=self.acl, nulls_ok=False)
+        entry, _acl = self.root.follow_acl(root, acl=self.acl, nulls_ok=False)
 
         self.root = entry
         self.is_chroot = True
@@ -763,6 +756,10 @@ class ServerClient:
 
         return await AuthList(self, msg)()
 
+    async def cmd_auth_info(self, msg):
+        msg["path"] = Path(None, "auth")
+        return await self.cmd_get_internal(msg)
+
     async def cmd_root(self, msg):
         """Change to a sub-tree.
         """
@@ -770,13 +767,13 @@ class ServerClient:
         return self.root.serialize(chop_path=self._chop_path, conv=self.conv)
 
     async def cmd_get_internal(self, msg):
-        return await self.cmd_get_value(msg, root=self.metaroot)
+        return await self.cmd_get_value(msg, root=self.metaroot, _nulls_ok=True)
 
     async def cmd_set_internal(self, msg):
-        return await self.cmd_set_value(msg, root=self.metaroot)
+        return await self.cmd_set_value(msg, root=self.metaroot, _nulls_ok=True)
 
     async def cmd_enum_internal(self, msg):
-        return await self.cmd_enum(msg, root=self.metaroot)
+        return await self.cmd_enum(msg, root=self.metaroot, _nulls_ok=True)
 
     async def cmd_delete_internal(self, msg):
         return await self.cmd_delete_value(msg, root=self.metaroot)
@@ -792,7 +789,7 @@ class ServerClient:
         acl2 = msg.get("acl", None)
         try:
             _entry, _acl = root.follow_acl(
-                *msg.path,
+                msg.path,
                 acl=self.acl,
                 acl_key="a" if acl2 is None else mode,
                 nulls_ok=False,
@@ -800,13 +797,11 @@ class ServerClient:
             )
 
             if acl2 is not None:
-                ok = acl.allows(  # pylint: disable=no-value-for-parameter # pylint is confused
-                    "a"
-                )
-                acl2 = root.follow(None, "acl", acl2, create=False, nulls_ok=True)
+                ok = acl.allows("a")  # pylint: disable=no-value-for-parameter # pylint is confused
+                acl2 = root.follow(Path(None, "acl", acl2), create=False, nulls_ok=True)
                 acl2 = ACLFinder(acl2)
                 _entry, acl = root.follow_acl(
-                    *msg.path, acl=acl2, acl_key=mode, nulls_ok=False, create=None
+                    msg.path, acl=acl2, acl_key=mode, nulls_ok=False, create=None
                 )
                 if not ok:
                     acl.block("a")
@@ -824,7 +819,7 @@ class ServerClient:
         if with_data is None:
             with_data = msg.get("with_data", False)
         entry, acl = root.follow_acl(
-            *msg.path, acl=self.acl, acl_key="e", create=False, nulls_ok=_nulls_ok
+            msg.path, acl=self.acl, acl_key="e", create=False, nulls_ok=_nulls_ok
         )
         empty = msg.get("empty", False)
         if with_data:
@@ -862,16 +857,14 @@ class ServerClient:
             root = self.root
         try:
             entry, _ = root.follow_acl(
-                *msg.path, create=False, acl=self.acl, acl_key="r", nulls_ok=_nulls_ok
+                msg.path, create=False, acl=self.acl, acl_key="r", nulls_ok=_nulls_ok
             )
         except KeyError:
             entry = {}
             if msg.get("nchain", 0):
                 entry["chain"] = None
         else:
-            entry = entry.serialize(
-                chop_path=-1, nchain=msg.get("nchain", 0), conv=self.conv
-            )
+            entry = entry.serialize(chop_path=-1, nchain=msg.get("nchain", 0), conv=self.conv)
         return entry
 
     async def cmd_set_value(self, msg, **kw):
@@ -899,14 +892,10 @@ class ServerClient:
             acl = self.acl
         else:
             acl = NullACL
-        entry, acl = root.follow_acl(
-            *msg.path, acl=acl, acl_key="W", nulls_ok=_nulls_ok
-        )
+        entry, acl = root.follow_acl(msg.path, acl=acl, acl_key="W", nulls_ok=_nulls_ok)
         if root is self.root and "match" in self.metaroot:
             try:
-                self.metaroot["match"].check_value(
-                    None if value is NotGiven else value, entry
-                )
+                self.metaroot["match"].check_value(None if value is NotGiven else value, entry)
             except ClientError:
                 raise
             except Exception as exc:
@@ -917,16 +906,16 @@ class ServerClient:
         send_prev = True
         if "prev" in msg:
             if entry.data != msg.prev:
-                raise ClientError("Data is %s" % (repr(entry.data),))
+                raise ClientError(f"Data is {entry.data !r}")
             send_prev = False
         if "chain" in msg:
             if msg.chain is None:
                 if entry.data is not NotGiven:
-                    raise ClientError("This entry already exists")
+                    raise ClientChainError("This entry already exists")
             elif entry.data is NotGiven:
-                raise ClientError("This entry is new")
+                raise ClientChainError("This entry is new")
             elif entry.chain != msg.chain:
-                raise ClientError("Chain is %s" % (repr(entry.chain),))
+                raise ClientChainError(f"Chain is {entry.chain !r}")
             send_prev = False
 
         res = attrdict()
@@ -942,9 +931,7 @@ class ServerClient:
         async with self.server.next_event() as event:
             await entry.set_data(
                 event,
-                NotGiven
-                if value is NotGiven
-                else self.conv.dec_value(value, entry=entry),
+                NotGiven if value is NotGiven else self.conv.dec_value(value, entry=entry),
                 server=self.server,
                 tock=self.server.tock,
             )
@@ -961,11 +948,7 @@ class ServerClient:
         You usually do this via a stream command.
         """
         msg = UpdateEvent.deserialize(
-            self.root,
-            msg,
-            nulls_ok=self.nulls_ok,
-            conv=self.conv,
-            cache=self.server._nodes,
+            self.root, msg, nulls_ok=self.nulls_ok, conv=self.conv, cache=self.server._nodes
         )
         res = await msg.entry.apply(msg, server=self, root=self.root)
         if res is None:
@@ -1018,7 +1001,7 @@ class ServerClient:
 
         try:
             entry, acl = self.root.follow_acl(
-                *msg.path, acl=self.acl, acl_key="d", nulls_ok=self.nulls_ok
+                msg.path, acl=self.acl, acl_key="d", nulls_ok=self.nulls_ok
             )
         except KeyError:
             return False
@@ -1027,15 +1010,10 @@ class ServerClient:
             res = 0
             if entry.data is not None and acl.allows("d"):
                 async with self.server.next_event() as event:
-                    evt = await entry.set_data(
-                        event, NotGiven, server=self, tock=self.server.tock
-                    )
+                    evt = await entry.set_data(event, NotGiven, server=self, tock=self.server.tock)
                     if nchain:
                         r = evt.serialize(
-                            chop_path=self._chop_path,
-                            nchain=nchain,
-                            with_old=True,
-                            conv=self.conv,
+                            chop_path=self._chop_path, nchain=nchain, with_old=True, conv=self.conv
                         )
                         r["seq"] = seq
                         r.pop("new_value", None)  # always None
@@ -1076,7 +1054,7 @@ class ServerClient:
     async def cmd_set_auth_typ(self, msg):
         if not self.user.is_super_root:
             raise RuntimeError("You're not allowed to do that")
-        a = self.root.follow(None, "auth", nulls_ok=True)
+        a = self.root.follow(Path(None, "auth"), nulls_ok=True)
         if a.data is NotGiven:
             val = {}
         else:
@@ -1085,9 +1063,7 @@ class ServerClient:
         if msg.typ is None:
             val.pop("current", None)
         elif msg.typ not in a or not len(a[msg.typ]["user"].keys()):
-            raise RuntimeError(
-                "You didn't configure this method yet:" + repr((msg.typ, vars(a)))
-            )
+            raise RuntimeError("You didn't configure this method yet:" + repr((msg.typ, vars(a))))
         else:
             val["current"] = msg.typ
         msg.value = val
@@ -1134,7 +1110,7 @@ class ServerClient:
                 "tock": self.server.tock,
             }
             try:
-                auth = self.root.follow(None, "auth", nulls_ok=True, create=False)
+                auth = self.root.follow(Path(None, "auth"), nulls_ok=True, create=False)
             except KeyError:
                 a = None
             else:
@@ -1167,15 +1143,17 @@ class ServerClient:
                         else:
                             if self.seq >= seq:
                                 raise ClientError(
-                                    "Channel closed? Sequence error: %d < %d"
-                                    % (self.seq, msg.seq)
+                                    f"Channel closed? Sequence error: {self.seq} < {msg.seq}"
                                 )
                             self.seq = seq
                             evt = anyio.create_event()
                             await self.tg.spawn(self.process, msg, evt)
                             await evt.wait()
                     except Exception as exc:
-                        if not isinstance(exc, ClientError):
+                        msg = {"error": str(exc)}
+                        if isinstance(exc, ClientError):  # pylint doesn't seem to see this
+                            msg["etype"] = exc.etype  # pylint: disable=no-member  ### YES IT HAS
+                        else:
                             self.logger.exception(
                                 "ERR %d: Client error on %s", self._client_nr, repr(msg)
                             )
@@ -1272,7 +1250,7 @@ class _RecoverControl:
 
 class Server:
     """
-    This is the DistKV server. It manages connections to the Serf server,
+    This is the DistKV server. It manages connections to the Serf/MQTT server,
     the DistKV clients, and (optionally) logs all changes to a file.
 
     Args:
@@ -1293,8 +1271,8 @@ class Server:
     _actor = None
     _del_actor = None
     cfg: attrdict = None
+    force_startup: bool = False
 
-    spawn: anyio.abc.TaskGroup = None
     seen_missing = None
     fetch_running = None
     sending_missing = None
@@ -1306,8 +1284,9 @@ class Server:
 
         self.cfg = combine_dict(cfg or {}, CFG, cls=attrdict)
         if isinstance(self.cfg.server.root, str):
-            self.cfg.server.root = self.cfg.server.root.split(".")
-        self.cfg.server.root = tuple(self.cfg.server.root)  # idempotent
+            self.cfg.server.root = P(self.cfg.server.root)
+        else:
+            self.cfg.server.root = Path.build(self.cfg.server.root)
 
         self.paranoid_root = self.root if self.cfg.server.paranoia else None
         self._nodes = {}
@@ -1345,14 +1324,11 @@ class Server:
                 yield n
             except BaseException as exc:
                 if n is not None:
-                    self.logger.warning(
-                        "Deletion %s %d due to %r", self.node, n.tick, exc
-                    )
+                    self.logger.warning("Deletion %s %d due to %r", self.node, n.tick, exc)
                     self.node.report_deleted(RangeSet((nt,)), self)
                     async with anyio.move_on_after(2, shield=True):
                         await self._send_event(
-                            "info",
-                            dict(node="", tick=0, deleted={self.node.name: (nt,)}),
+                            "info", dict(node="", tick=0, deleted={self.node.name: (nt,)})
                         )
                 raise
             finally:
@@ -1483,10 +1459,7 @@ class Server:
                     async def send_nodes():
                         nonlocal nodes, n_nodes
                         await client._request(  # pylint: disable=cell-var-from-loop
-                            "check_deleted",
-                            iter=False,
-                            nchain=-1,
-                            nodes=nodes.serialize(),
+                            "check_deleted", iter=False, nchain=-1, nodes=nodes.serialize()
                         )
                         nodes.clear()
                         n_nodes = 0
@@ -1536,6 +1509,8 @@ class Server:
         deleted=False,
         missing=False,
         present=False,
+        debug=False,
+        debugger=False,
         remote_missing=False,
         **_kw,
     ):
@@ -1582,6 +1557,18 @@ class Server:
                 lk = n.remote_missing
                 if len(lk):
                     nd[n.name] = lk.__getstate__()
+        if debug:
+            nd = res.debug = attrdict()
+            # TODO insert some debugging info
+
+        if debugger:
+            try:
+                import pdb_clone as pdb
+            except ImportError:
+                res["debugger"] = "Import error"
+            else:
+                pdb().set_trace_remote(host=b"127.0.0.1", port=57935)
+
         res["node"] = self.node.name
         res["tock"] = self.tock
         return res
@@ -1684,12 +1671,12 @@ class Server:
         # TODO use a msgpack extension instead
         if isinstance(msg, Mapping):
             i = 0
-            while ("_p%d" % i) in msg:
+            while (f"_p{i}") in msg:
                 i += 1
             j = i
             while i:
                 i -= 1
-                msg["_p%d" % (i + 1)] = msg["_p%d" % i]
+                msg[f"_p{i+1}"] = msg[f"_p{i}"]
             if j:
                 msg["_p0"] = ""
 
@@ -1709,7 +1696,7 @@ class Server:
             return
         yield p
 
-    async def _unpack_multiple(self, msg):
+    def _unpack_multiple(self, msg):
         """
         Undo the effects of _pack_multiple.
         """
@@ -1731,12 +1718,14 @@ class Server:
                     return None
                 p = b"".join(s)
                 del self._part_cache[(nn, seq)]
+                msg = unpacker(p)
+                msg["_p0"] = ""
 
             i = 0
-            while ("_p%d" % (i + 1)) in msg:
-                msg["_p%d" % i] = msg["_p%d" % (i + 1)]
+            while f"_p{i+1}" in msg:
+                msg[f"_p{i}"] = msg[f"_p{i+1}"]
                 i += 1
-            del msg["_p%d" % i]
+            del msg[f"_p{i}"]
         return msg
 
     async def monitor(self, action: str, delay: anyio.abc.Event = None):
@@ -1757,14 +1746,21 @@ class Server:
 
                 async for resp in stream:
                     msg = unpacker(resp.payload)
-                    msg = await self._unpack_multiple(msg)
+                    msg = self._unpack_multiple(msg)
                     if not msg:  # None, empty, whatever
                         continue
                     self.logger.debug("Recv %s: %r", action, msg)
-                    await self.tock_seen(msg.get("tock", 0))
-                    await cmd(msg)
-        except CancelledError:
-            pass
+                    async with anyio.fail_after(10):
+                        await self.tock_seen(msg.get("tock", 0))
+                        await cmd(msg)
+        except (CancelledError, trioCancelled):
+            self.logger.warning("Cancelled %s", action)
+            raise
+        except BaseException as exc:
+            self.logger.exception("Died %s: %r", action, exc)
+            raise
+        else:
+            self.logger.error("Stream ended %s", action)
 
     async def _run_del(self, evt):
         try:
@@ -1797,6 +1793,7 @@ class Server:
             await delay.set()
             async for msg in actor:
                 # self.logger.debug("IN %r",msg)
+
                 if isinstance(msg, RecoverEvent):
                     await self.spawn(
                         self.recover_split,
@@ -1805,8 +1802,10 @@ class Server:
                         msg.local_nodes,
                         msg.remote_nodes,
                     )
+
                 elif isinstance(msg, GoodNodeEvent):
                     await self.spawn(self.fetch_data, msg.nodes)
+
                 elif isinstance(msg, RawMsgEvent):
                     msg = msg.msg
                     msg_node = msg.get("node", None)
@@ -1819,6 +1818,11 @@ class Server:
                         await self.tock_seen(val[0])
                         val = val[1]
                     Node(msg_node, val, cache=self._nodes)
+
+                elif isinstance(msg, TagEvent):
+                    # We're "it"; find missing data
+                    await self._send_missing()
+
                 elif isinstance(msg, (TagEvent, UntagEvent, DetagEvent)):
                     pass
                     # TODO tell clients, for cleanup tasks in handlers,
@@ -1836,7 +1840,7 @@ class Server:
         try:
             # First try to read the host name from the meta-root's
             # "hostmap" entry, if any.
-            hme = self.root.follow(None, "hostmap", host, create=False, nulls_ok=True)
+            hme = self.root.follow(Path(None, "hostmap", host), create=False, nulls_ok=True)
             if hme.data is NotGiven:
                 raise KeyError(host)
         except KeyError:
@@ -1905,7 +1909,7 @@ class Server:
             await self._check_ticked()
         self.fetch_running = None
 
-    async def fetch_data(self, nodes):
+    async def fetch_data(self, nodes, authoritative=False):
         """
         We are newly started and don't have any data.
 
@@ -1932,17 +1936,11 @@ class Server:
 
                     pl = PathLongener(())
                     res = await client._request(
-                        "get_tree",
-                        iter=True,
-                        from_server=self.node.name,
-                        nchain=-1,
-                        path=(),
+                        "get_tree", iter=True, from_server=self.node.name, nchain=-1, path=()
                     )
                     async for r in res:
                         pl(r)
-                        r = UpdateEvent.deserialize(
-                            self.root, r, cache=self._nodes, nulls_ok=True
-                        )
+                        r = UpdateEvent.deserialize(self.root, r, cache=self._nodes, nulls_ok=True)
                         await r.entry.apply(r, server=self, root=self.paranoid_root)
                     await self.tock_seen(res.end_msg.tock)
 
@@ -1956,9 +1954,7 @@ class Server:
                     )
                     async for r in res:
                         pl(r)
-                        r = UpdateEvent.deserialize(
-                            self.root, r, cache=self._nodes, nulls_ok=True
-                        )
+                        r = UpdateEvent.deserialize(self.root, r, cache=self._nodes, nulls_ok=True)
                         await r.entry.apply(r, server=self, root=self.paranoid_root)
                     await self.tock_seen(res.end_msg.tock)
 
@@ -1981,13 +1977,16 @@ class Server:
                 # node's state, so we now need to find whatever that
                 # node didn't have.
 
+                if authoritative:
+                    # … or not.
+                    self._discard_all_missing()
                 for nst in self._nodes.values():
                     if nst.tick and len(nst.local_missing):
                         self.fetch_missing.add(nst)
                 if len(self.fetch_missing):
                     self.fetch_running = False
                     await self.spawn(self.do_send_missing)
-                else:
+                if self.force_startup or not len(self.fetch_missing):
                     if self.node.tick is None:
                         self.node.tick = 0
                     self.fetch_running = None
@@ -2083,24 +2082,7 @@ class Server:
                     await t.wait(2)
 
                 if x.cancel_called:
-                    msg = dict()
-                    for n in list(self._nodes.values()):
-                        if not n.tick:
-                            continue
-                        m = n.local_missing
-                        mr = self.seen_missing.get(n.name, None)
-                        if mr is not None:
-                            m -= mr
-                        if len(m) == 0:
-                            continue
-                        msg[n.name] = m.__getstate__()
-                        if mr is None:
-                            self.seen_missing[n.name] = m
-                        else:
-                            mr += m
-
-                    msg = attrdict(missing=msg)
-                    await self._send_event("info", msg)
+                    await self._send_missing(force=True)
 
                 # wait a bit more before continuing. Again this depends on
                 # `prio` so that there won't be two nodes that send the same
@@ -2111,13 +2093,34 @@ class Server:
                 await self._run_send_missing(prio)
 
             finally:
-                async with anyio.fail_after(2, shield=True):
+                async with anyio.open_cancel_scope(shield=True):
                     # Protect against cleaning up when another recovery task has
                     # been started (because we saw another merge)
                     self.logger.debug("SplitRecover %d: finished @%d", t._id, t.tock)
                     self.seen_missing = {}
                     await t.cancel()
 
+    async def _send_missing(self, force=False):
+        msg = dict()
+        for n in list(self._nodes.values()):
+            if not n.tick:
+                continue
+            m = n.local_missing
+            mr = self.seen_missing.get(n.name, None)
+            if mr is not None:
+                m -= mr
+            if len(m) == 0:
+                continue
+            msg[n.name] = m.__getstate__()
+            if mr is None:
+                self.seen_missing[n.name] = m
+            else:
+                mr += m
+
+        if force or msg:
+            msg = attrdict(missing=msg)
+            await self._send_event("info", msg)
+
     async def _run_send_missing(self, prio):
         """Start :meth:`_send_missing_data` if it's not running"""
 
@@ -2139,9 +2142,7 @@ class Server:
         if prio is None:
             await anyio.sleep(clock * (1 + self._actor.random / 3))
         else:
-            await anyio.sleep(
-                clock * (1 - (1 / (1 << prio)) / 2 - self._actor.random / 5)
-            )
+            await anyio.sleep(clock * (1 - (1 / (1 << prio)) / 2 - self._actor.random / 5))
 
         self.logger.debug("SendMissingGo %s %s", prio, self.sending_missing)
         while self.sending_missing:
@@ -2152,10 +2153,7 @@ class Server:
             deleted = {}
             for n in nodes:
                 self.logger.debug(
-                    "SendMissingGo %s %r %r",
-                    n.name,
-                    n.remote_missing,
-                    n.local_superseded,
+                    "SendMissingGo %s %r %r", n.name, n.remote_missing, n.local_superseded
                 )
                 k = n.remote_missing & n.local_superseded
                 for r in n.remote_missing & n.local_present:
@@ -2181,7 +2179,11 @@ class Server:
         self.sending_missing = None
 
     async def load(
-        self, path: str = None, stream: io.IOBase = None, local: bool = False
+        self,
+        path: str = None,
+        stream: io.IOBase = None,
+        local: bool = False,
+        authoritative: bool = False,
     ):
         """Load data from this stream
 
@@ -2205,17 +2207,28 @@ class Server:
                         await self.tock_seen(m.tock)
                     else:
                         m.tock = self.tock
-                    m = UpdateEvent.deserialize(
-                        self.root, m, cache=self._nodes, nulls_ok=True
-                    )
+                    m = UpdateEvent.deserialize(self.root, m, cache=self._nodes, nulls_ok=True)
                     await self.tock_seen(m.tock)
-                    await m.entry.apply(m, server=self, root=self.paranoid_root)
+                    await m.entry.apply(m, server=self, root=self.paranoid_root, loading=True)
                 elif "nodes" in m or "known" in m or "deleted" in m or "tock" in m:
                     await self._process_info(m)
                 else:
                     self.logger.warning("Unknown message in stream: %s", repr(m))
+
+        if authoritative:
+            self._discard_all_missing()
+
         self.logger.debug("Loading finished.")
 
+    def _discard_all_missing(self):
+        for n in self._nodes.values():
+            if not n.tick:
+                continue
+            lk = n.local_missing
+
+            if len(lk):
+                n.report_superseded(lk, local=True)
+
     async def _save(self, writer, shorter, nchain=-1, full=False):
         """Save the current state.
         """
@@ -2320,22 +2333,17 @@ class Server:
             self._savers.append(state)
             try:
                 await self.save_stream(
-                    path=path,
-                    stream=stream,
-                    done=done,
-                    done_val=s,
-                    save_state=save_state,
+                    path=path, stream=stream, done=done, done_val=s, save_state=save_state
                 )
             except EnvironmentError as err:
                 if done is None:
                     raise
                 await done.set_error(err)
             finally:
-                await sd.set()
+                async with anyio.open_cancel_scope(shield=True):
+                    await sd.set()
 
-    async def run_saver(
-        self, path: str = None, stream=None, save_state=False, wait: bool = True
-    ):
+    async def run_saver(self, path: str = None, stream=None, save_state=False, wait: bool = True):
         """
         Start a task that continually saves to disk.
 
@@ -2356,13 +2364,7 @@ class Server:
         res = None
         if path is not None:
             await self.spawn(
-                partial(
-                    self._saver,
-                    path=path,
-                    stream=stream,
-                    save_state=save_state,
-                    done=done,
-                )
+                partial(self._saver, path=path, stream=stream, save_state=save_state, done=done)
             )
             if wait:
                 res = await done.get()
@@ -2392,14 +2394,16 @@ class Server:
         """Await this to determine if/when the server is serving clients."""
         await self._ready2.wait()
 
-    async def serve(self, log_path=None, log_inc=False, ready_evt=None):
+    async def serve(self, log_path=None, log_inc=False, force=False, ready_evt=None):
         """Task that opens a Serf connection and actually runs the server.
 
         Args:
           ``setup_done``: optional event that's set when the server is initially set up.
           ``log_path``: path to a binary file to write changes and initial state to.
           ``log_inc``: if saving, write changes, not the whole state.
+          ``force``: start up even if entries are missing
         """
+        self.force_startup = force
         back = get_backend(self.cfg.server.backend)
         try:
             conn = self.cfg.server[self.cfg.server.backend]
@@ -2469,9 +2473,7 @@ class Server:
                 assert self.node.tick is None
                 self.node.tick = 0
                 async with self.next_event() as event:
-                    await self.root.set_data(
-                        event, self._init, tock=self.tock, server=self
-                    )
+                    await self.root.set_data(event, self._init, tock=self.tock, server=self)
 
             await self.spawn(self._sigterm)
 
@@ -2546,6 +2548,7 @@ class Server:
             except Exception:
                 pass
         finally:
-            if c is not None:
-                self._clients.remove(c)
-            await stream.aclose()
+            async with anyio.move_on_after(2, shield=True):
+                if c is not None:
+                    self._clients.remove(c)
+                await stream.aclose()
