@@ -2,10 +2,10 @@ import pytest
 import trio
 import anyio
 
-from .mock_mqtt import stdtest
-from .run import run
+from distkv.mock.mqtt import stdtest
+
 from distkv.client import ServerError
-from distkv.util import PathLongener
+from distkv.util import PathLongener, P
 from functools import partial
 
 import logging
@@ -23,7 +23,7 @@ async def test_21_load_save(autojump_clock, tmpdir):  # pylint: disable=unused-a
 
     async def watch_changes(c, evt):
         lg = PathLongener(())
-        async with c.watch(nchain=3, fetch=True) as res:
+        async with c.watch(P(":"), nchain=3, fetch=True) as res:
             await evt.set()
             async for m in res:
                 logger.info(m)
@@ -32,16 +32,16 @@ async def test_21_load_save(autojump_clock, tmpdir):  # pylint: disable=unused-a
                     msgs.append(m)
 
     async with stdtest(args={"init": 234}, tocks=30) as st:
-        s, = st.s
+        (s,) = st.s
         async with st.client() as c:
-            assert (await c.get()).value == 234
+            assert (await c.get(P(":"))).value == 234
             evt = anyio.create_event()
             await c.tg.spawn(watch_changes, c, evt)
             await evt.wait()
 
-            await c.set("foo", value="hello", nchain=3)
-            await c.set("foo", "bar", value="baz", nchain=3)
-            await c.set(value=2345, nchain=3)
+            await c.set(P("foo"), value="hello", nchain=3)
+            await c.set(P("foo.bar"), value="baz", nchain=3)
+            await c.set(P(":"), value=2345, nchain=3)
             await trio.sleep(1)  # allow the writer to write
             pass  # client end
 
@@ -55,31 +55,19 @@ async def test_21_load_save(autojump_clock, tmpdir):  # pylint: disable=unused-a
         m.pop("tock", None)
         m.pop("seq", None)
     assert sorted(msgs, key=lambda x: x.chain.tick if "chain" in x else 0) == [
-        {
-            "chain": {"node": "test_0", "prev": None, "tick": 1},
-            "path": (),
-            "value": 234,
-        },
-        {
-            "chain": {"node": "test_0", "prev": None, "tick": 2},
-            "path": ("foo",),
-            "value": "hello",
-        },
+        {"chain": {"node": "test_0", "prev": None, "tick": 1}, "path": P(":"), "value": 234},
+        {"chain": {"node": "test_0", "prev": None, "tick": 2}, "path": P("foo"), "value": "hello"},
         {
             "chain": {"node": "test_0", "prev": None, "tick": 3},
-            "path": ("foo", "bar"),
+            "path": P("foo.bar"),
             "value": "baz",
         },
-        {
-            "chain": {"node": "test_0", "prev": None, "tick": 4},
-            "path": (),
-            "value": 2345,
-        },
+        {"chain": {"node": "test_0", "prev": None, "tick": 4}, "path": P(":"), "value": 2345},
     ]
 
     msgs = []
     async with stdtest(run=False, tocks=40) as st:
-        s, = st.s
+        (s,) = st.s
         logger.debug("LOAD %s", path)
         await s.load(path, local=True)
         logger.debug("LOADED")
@@ -95,34 +83,26 @@ async def test_21_load_save(autojump_clock, tmpdir):  # pylint: disable=unused-a
             await c.tg.spawn(watch_changes, c, evt)
             await evt.wait()
 
-            await c.set("foof", value="again")
-            assert (await c.get("foo")).value == "hello"
-            assert (await c.get("foo", "bar")).value == "baz"
-            assert (await c.get()).value == 2345
+            await c.set(P("foof"), value="again")
+            assert (await c.get(P("foo"))).value == "hello"
+            assert (await c.get(P("foo.bar"))).value == "baz"
+            assert (await c.get(P(":"))).value == 2345
             await trio.sleep(1)  # allow the writer to write
 
     for m in msgs:
         m.pop("tock", None)
         m.pop("seq", None)
     assert sorted(msgs, key=lambda x: x.chain.tick if "chain" in x else 0) == [
-        {
-            "chain": {"node": "test_0", "prev": None, "tick": 2},
-            "path": ("foo",),
-            "value": "hello",
-        },
+        {"chain": {"node": "test_0", "prev": None, "tick": 2}, "path": P("foo"), "value": "hello"},
         {
             "chain": {"node": "test_0", "prev": None, "tick": 3},
-            "path": ("foo", "bar"),
+            "path": P("foo.bar"),
             "value": "baz",
         },
-        {
-            "chain": {"node": "test_0", "prev": None, "tick": 4},
-            "path": (),
-            "value": 2345,
-        },
+        {"chain": {"node": "test_0", "prev": None, "tick": 4}, "path": P(":"), "value": 2345},
         {
             "chain": {"node": "test_0", "prev": None, "tick": 5},
-            "path": ("foof",),
+            "path": P("foof"),
             "value": "again",
         },
     ]
@@ -132,28 +112,18 @@ async def test_21_load_save(autojump_clock, tmpdir):  # pylint: disable=unused-a
 @pytest.mark.trio
 async def test_02_cmd(autojump_clock):  # pylint: disable=unused-argument
     async with stdtest(args={"init": 123}, tocks=50) as st:
-        s, = st.s
         async with st.client() as c:
-            assert (await c.get()).value == 123
-            h = p = None  # pylint
-            for h, p, *_ in s.ports:
-                if h[0] != ":":
-                    break
+            assert (await c.get(P(":"))).value == 123
+            r = await st.run("data set -v hello foo")
+            r = await st.run("data set -ev 'baz' foo.bar")
 
-            r = await run(
-                "client", "-h", h, "-p", p, "data", "set", "-v", "hello", "foo"
-            )
-            r = await run(
-                "client", "-h", h, "-p", p, "data", "set", "-ev", "'baz'", "foo", "bar"
-            )
-
-            r = await run("client", "-h", h, "-p", p, "data", "get")
+            r = await st.run("data get :")
             assert r.stdout == "123\n"
 
-            r = await run("client", "-h", h, "-p", p, "data", "get", "foo")
+            r = await st.run("data get foo")
             assert r.stdout == "'hello'\n"
 
-            r = await run("client", "-h", h, "-p", p, "data", "get", "foo", "bar")
+            r = await st.run("data get foo.bar")
             assert r.stdout == "'baz'\n"
 
             r = await c._request(
@@ -176,12 +146,10 @@ async def test_02_cmd(autojump_clock):  # pylint: disable=unused-argument
             }
 
             assert (await c._request("get_value", node="test_0", tick=1)).value == 123
-            assert (
-                await c._request("get_value", node="test_0", tick=2)
-            ).value == "hello"
+            assert (await c._request("get_value", node="test_0", tick=2)).value == "hello"
             assert (await c._request("get_value", node="test_0", tick=3)).value == "baz"
 
-            r = await c.set(value=1234, nchain=3)
+            r = await c.set(P(":"), value=1234, nchain=3)
             assert r.prev == 123
             assert r.chain.tick == 4
 
@@ -220,7 +188,7 @@ async def test_02_cmd(autojump_clock):  # pylint: disable=unused-argument
 async def test_03_three(autojump_clock):  # pylint: disable=unused-argument
     async with stdtest(test_1={"init": 125}, n=2, tocks=30) as st:
         async with st.client(1) as ci:
-            assert (await ci.get()).value == 125
+            assert (await ci.get(P(":"))).value == 125
 
             r = await ci._request(
                 "get_state",
@@ -312,9 +280,9 @@ async def test_03_three(autojump_clock):  # pylint: disable=unused-argument
                     "remote_missing": {},
                 }
 
-                assert (await c.get()).value == 125
+                assert (await c.get(P(":"))).value == 125
 
-                r = await c.set(value=126, nchain=3)
+                r = await c.set(P(":"), value=126, nchain=3)
                 assert r.prev == 125
                 assert r.chain.tick == 1
                 assert r.chain.node == "test_0"
@@ -326,7 +294,7 @@ async def test_03_three(autojump_clock):  # pylint: disable=unused-argument
                 # and the initial change is no longer retrievable.
                 # We need the latter to ensure that there are no memory leaks.
                 await trio.sleep(1)
-                r = await ci.set(value=127, nchain=3)
+                r = await ci.set(P(":"), value=127, nchain=3)
                 assert r.prev == 126
                 assert r.chain.tick == 2
                 assert r.chain.node == "test_1"
